@@ -2,8 +2,8 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2015-2019 Takao Fujiwara <takao.fujiwara1@gmail.com>
- * Copyright (C) 2008-2019 Red Hat, Inc.
+ * Copyright (C) 2015-2021 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2008-2021 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,8 +32,19 @@
 #include "ibusimcontext.h"
 
 #ifdef GDK_WINDOWING_WAYLAND
+#if GTK_CHECK_VERSION (3, 98, 4)
+#include <gdk/wayland/gdkwayland.h>
+#else
 #include <gdk/gdkwayland.h>
 #endif
+#endif
+
+#ifdef GDK_WINDOWING_X11
+#if GTK_CHECK_VERSION (3, 98, 4)
+#include <gdk/x11/gdkx.h>
+#include <X11/Xlib.h>
+#endif
+#endif
 
 #if !GTK_CHECK_VERSION (2, 91, 0)
 #  define DEPRECATED_GDK_KEYSYMS 1
@@ -52,7 +63,11 @@ struct _IBusIMContext {
 
     /* instance members */
     GtkIMContext *slave;
+#if GTK_CHECK_VERSION (3, 98, 4)
+    GtkWidget *client_window;
+#else
     GdkWindow *client_window;
+#endif
 
     IBusInputContext *ibuscontext;
 
@@ -73,7 +88,12 @@ struct _IBusIMContext {
     GCancellable    *cancellable;
     GQueue          *events_queue;
 
-#if !GTK_CHECK_VERSION (3, 93, 0)
+#if GTK_CHECK_VERSION (3, 98, 4)
+    GdkSurface      *surface;
+    GdkDevice       *device;
+    double           x;
+    double           y;
+#else
     gboolean         use_button_press_event;
 #endif
 };
@@ -90,19 +110,25 @@ static guint    _signal_preedit_end_id = 0;
 static guint    _signal_delete_surrounding_id = 0;
 static guint    _signal_retrieve_surrounding_id = 0;
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+static gboolean _use_sync_mode = TRUE;
+#else
 static const gchar *_no_snooper_apps = NO_SNOOPER_APPS;
 static gboolean _use_key_snooper = ENABLE_SNOOPER;
 static guint    _key_snooper_id = 0;
 
 static gboolean _use_sync_mode = FALSE;
+#endif
 
 static const gchar *_discard_password_apps  = "";
 static gboolean _use_discard_password = FALSE;
 
 static GtkIMContext *_focus_im_context = NULL;
 static IBusInputContext *_fake_context = NULL;
+#if !GTK_CHECK_VERSION (3, 98, 4)
 static GdkWindow *_input_window = NULL;
 static GtkWidget *_input_widget = NULL;
+#endif
 
 /* functions prototype */
 static void     ibus_im_context_class_init  (IBusIMContextClass    *class);
@@ -114,7 +140,11 @@ static void     ibus_im_context_finalize    (GObject               *obj);
 static void     ibus_im_context_reset       (GtkIMContext          *context);
 static gboolean ibus_im_context_filter_keypress
                                             (GtkIMContext           *context,
+#if GTK_CHECK_VERSION (3, 98, 4)
+                                             GdkEvent               *key);
+#else
                                              GdkEventKey            *key);
+#endif
 static void     ibus_im_context_focus_in    (GtkIMContext          *context);
 static void     ibus_im_context_focus_out   (GtkIMContext          *context);
 static void     ibus_im_context_get_preedit_string
@@ -122,20 +152,34 @@ static void     ibus_im_context_get_preedit_string
                                              gchar                  **str,
                                              PangoAttrList          **attrs,
                                              gint                   *cursor_pos);
+#if GTK_CHECK_VERSION (3, 98, 4)
+static void     ibus_im_context_set_client_widget
+                                            (GtkIMContext           *context,
+                                             GtkWidget              *client);
+#else
 static void     ibus_im_context_set_client_window
                                             (GtkIMContext           *context,
                                              GdkWindow              *client);
+#endif
 static void     ibus_im_context_set_cursor_location
                                             (GtkIMContext           *context,
                                              GdkRectangle           *area);
 static void     ibus_im_context_set_use_preedit
                                             (GtkIMContext           *context,
                                              gboolean               use_preedit);
+#if !GTK_CHECK_VERSION (4, 1, 2)
 static void     ibus_im_context_set_surrounding
                                             (GtkIMContext  *slave,
                                              const gchar   *text,
-                                             gint           len,
-                                             gint           cursor_index);
+                                             int            len,
+                                             int            cursor_index);
+#endif
+static void     ibus_im_context_set_surrounding_with_selection
+                                            (GtkIMContext  *slave,
+                                             const gchar   *text,
+                                             int            len,
+                                             int            cursor_index,
+                                             int            anchor_index);
 
 /* static methods*/
 static void     _ibus_context_update_preedit_text_cb
@@ -239,6 +283,7 @@ ibus_im_context_new (void)
     return IBUS_IM_CONTEXT (obj);
 }
 
+#if !GTK_CHECK_VERSION (3, 98, 4)
 static gboolean
 _focus_in_cb (GtkWidget     *widget,
               GdkEventFocus *event,
@@ -260,22 +305,41 @@ _focus_out_cb (GtkWidget     *widget,
     }
     return FALSE;
 }
+#endif /* end of GTK_CHECK_VERSION (3, 98, 4) */
 
 static gboolean
 ibus_im_context_commit_event (IBusIMContext *ibusimcontext,
+#if GTK_CHECK_VERSION (3, 98, 4)
+                              GdkEvent      *event)
+#else
                               GdkEventKey   *event)
+#endif
 {
+    guint keyval = 0;
+    GdkModifierType state = 0;
     int i;
     GdkModifierType no_text_input_mask;
     gunichar ch;
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+    if (gdk_event_get_event_type (event) == GDK_KEY_RELEASE)
+        return FALSE;
+    keyval = gdk_key_event_get_keyval (event);
+    state = gdk_event_get_modifier_state (event);
+#else
     if (event->type == GDK_KEY_RELEASE)
         return FALSE;
+    keyval = event->keyval;
+    state = event->state;
+#endif
+
     /* Ignore modifier key presses */
     for (i = 0; i < G_N_ELEMENTS (IBUS_COMPOSE_IGNORE_KEYLIST); i++)
-        if (event->keyval == IBUS_COMPOSE_IGNORE_KEYLIST[i])
+        if (keyval == IBUS_COMPOSE_IGNORE_KEYLIST[i])
             return FALSE;
-#if GTK_CHECK_VERSION (3, 4, 0)
+#if GTK_CHECK_VERSION (3, 98, 4)
+    no_text_input_mask = GDK_MODIFIER_MASK;
+#elif GTK_CHECK_VERSION (3, 4, 0)
     no_text_input_mask = gdk_keymap_get_modifier_mask (
             gdk_keymap_get_for_display (gdk_display_get_default ()),
             GDK_MODIFIER_INTENT_NO_TEXT_INPUT);
@@ -290,31 +354,49 @@ ibus_im_context_commit_event (IBusIMContext *ibusimcontext,
 
 #  undef _IBUS_NO_TEXT_INPUT_MOD_MASK
 #endif
-    if (event->state & no_text_input_mask ||
-        event->keyval == GDK_KEY_Return ||
-        event->keyval == GDK_KEY_ISO_Enter ||
-        event->keyval == GDK_KEY_KP_Enter) {
+    if (state & no_text_input_mask ||
+        keyval == GDK_KEY_Return ||
+        keyval == GDK_KEY_ISO_Enter ||
+        keyval == GDK_KEY_KP_Enter) {
         return FALSE;
     }
-    ch = ibus_keyval_to_unicode (event->keyval);
+    ch = ibus_keyval_to_unicode (keyval);
     if (ch != 0 && !g_unichar_iscntrl (ch)) {
         IBusText *text = ibus_text_new_from_unichar (ch);
         g_signal_emit (ibusimcontext, _signal_commit_id, 0, text->text);
         g_object_unref (text);
         _request_surrounding_text (ibusimcontext);
+#if !GTK_CHECK_VERSION (3, 98, 4)
+        /* Avoid a loop with _ibus_context_forward_key_event_cb() */
+        event->state |= IBUS_HANDLED_MASK;
+#endif
         return TRUE;
     }
    return FALSE;
 }
 
+struct _ProcessKeyEventData {
+    GdkEvent *event;
+    IBusIMContext *ibusimcontext;
+};
+
+typedef struct _ProcessKeyEventData ProcessKeyEventData;
+
 static void
 _process_key_event_done (GObject      *object,
                          GAsyncResult *res,
                          gpointer      user_data)
 {
     IBusInputContext *context = (IBusInputContext *)object;
-    GdkEventKey *event = (GdkEventKey *) user_data;
+
+    ProcessKeyEventData *data = (ProcessKeyEventData *)user_data;
+    GdkEvent *event = data->event;
+#if GTK_CHECK_VERSION (3, 98, 4)
+    IBusIMContext *ibusimcontext = data->ibusimcontext;
+#endif
     GError *error = NULL;
+
+    g_slice_free (ProcessKeyEventData, data);
     gboolean retval = ibus_input_context_process_key_event_async_finish (
             context,
             res,
@@ -326,46 +408,96 @@ _process_key_event_done (GObject      *object,
     }
 
     if (retval == FALSE) {
-        event->state |= IBUS_IGNORED_MASK;
-        gdk_event_put ((GdkEvent *)event);
+#if GTK_CHECK_VERSION (3, 98, 4)
+        g_return_if_fail (GTK_IS_IM_CONTEXT (ibusimcontext));
+        gtk_im_context_filter_key (
+                GTK_IM_CONTEXT (ibusimcontext),
+                gdk_event_get_event_type (event) == GDK_KEY_PRESS,
+                gdk_event_get_surface (event),
+                gdk_event_get_device (event),
+                gdk_event_get_time (event),
+                gdk_key_event_get_keycode (event),
+                gdk_event_get_modifier_state (event) | IBUS_IGNORED_MASK,
+                0);
+#else
+        ((GdkEventKey *)event)->state |= IBUS_IGNORED_MASK;
+        gdk_event_put (event);
+#endif
     }
-    gdk_event_free ((GdkEvent *)event);
+#if GTK_CHECK_VERSION (3, 98, 4)
+    gdk_event_unref (event);
+#else
+    gdk_event_free (event);
+#endif
 }
 
 static gboolean
 _process_key_event (IBusInputContext *context,
-                    GdkEventKey      *event)
+#if GTK_CHECK_VERSION (3, 98, 4)
+                    GdkEvent         *event,
+#else
+                    GdkEventKey      *event,
+#endif
+                    IBusIMContext    *ibusimcontext)
 {
-    guint state = event->state;
+    guint state;
+    guint keyval = 0;
+    guint16 hardware_keycode = 0;
+    guint keycode = 0;
     gboolean retval = FALSE;
 
-    if (event->type == GDK_KEY_RELEASE) {
+#if GTK_CHECK_VERSION (3, 98, 4)
+    GdkModifierType gdkstate = gdk_event_get_modifier_state (event);
+    state = (uint)gdkstate;
+    if (gdk_event_get_event_type (event) == GDK_KEY_RELEASE)
         state |= IBUS_RELEASE_MASK;
-    }
+    keyval = gdk_key_event_get_keyval (event);
+    hardware_keycode = gdk_key_event_get_keycode (event);
+#else
+    state = event->state;
+    if (event->type == GDK_KEY_RELEASE)
+        state |= IBUS_RELEASE_MASK;
+    keyval = event->keyval;
+    hardware_keycode = event->hardware_keycode;
+#endif
+    keycode = hardware_keycode;
 
     if (_use_sync_mode) {
         retval = ibus_input_context_process_key_event (context,
-            event->keyval,
-            event->hardware_keycode - 8,
+            keyval,
+            keycode - 8,
             state);
     }
     else {
+        ProcessKeyEventData *data = g_slice_new0 (ProcessKeyEventData);
+#if GTK_CHECK_VERSION (3, 98, 4)
+        data->event = gdk_event_ref (event);
+#else
+        data->event = gdk_event_copy ((GdkEvent *)event);
+#endif
+        data->ibusimcontext = ibusimcontext;
         ibus_input_context_process_key_event_async (context,
-            event->keyval,
-            event->hardware_keycode - 8,
+            keyval,
+            keycode - 8,
             state,
             -1,
             NULL,
             _process_key_event_done,
-            gdk_event_copy ((GdkEvent *) event));
+            data);
 
         retval = TRUE;
     }
 
+    /* GTK4 does not provide gtk_key_snooper_install() and also
+     * GtkIMContextClass->filter_keypress() cannot send the updated
+     * GdkEventKey so event->state is not updated here in GTK4.
+     */
+#if !GTK_CHECK_VERSION (3, 98, 4)
     if (retval)
         event->state |= IBUS_HANDLED_MASK;
     else
         event->state |= IBUS_IGNORED_MASK;
+#endif
 
     return retval;
 }
@@ -425,6 +557,7 @@ _set_content_type (IBusIMContext *context)
 }
 
 
+#if !GTK_CHECK_VERSION (3, 98, 4)
 static gint
 _key_snooper_cb (GtkWidget   *widget,
                  GdkEventKey *event,
@@ -514,21 +647,24 @@ _key_snooper_cb (GtkWidget   *widget,
 
     } while (0);
 
-    if (ibusimcontext != NULL) {
+    if (ibusimcontext != NULL && event->type == GDK_KEY_PRESS) {
         /* "retrieve-surrounding" signal sometimes calls unref by
          * gtk_im_multicontext_get_slave() because priv->context_id is not
          * the latest than global_context_id in GtkIMMulticontext.
          * Since _focus_im_context is gotten by the focus_in event,
          * it would be good to call ref here.
+         *
+         * Most release key events would be redundant from
+         * _ibus_context_forward_key_event_cb ().
          */
         g_object_ref (ibusimcontext);
         _request_surrounding_text (ibusimcontext);
         ibusimcontext->time = event->time;
     }
 
-    retval = _process_key_event (ibuscontext, event);
+    retval = _process_key_event (ibuscontext, event, ibusimcontext);
 
-    if (ibusimcontext != NULL) {
+    if (ibusimcontext != NULL && event->type == GDK_KEY_PRESS) {
         /* unref ibusimcontext could call ibus_im_context_finalize here
          * because "retrieve-surrounding" signal could call unref.
          */
@@ -537,6 +673,7 @@ _key_snooper_cb (GtkWidget   *widget,
 
     return retval;
 }
+#endif
 
 static gboolean
 _get_boolean_env(const gchar *name,
@@ -599,10 +736,19 @@ ibus_im_context_class_init (IBusIMContextClass *class)
     im_context_class->focus_out = ibus_im_context_focus_out;
     im_context_class->filter_keypress = ibus_im_context_filter_keypress;
     im_context_class->get_preedit_string = ibus_im_context_get_preedit_string;
+#if GTK_CHECK_VERSION (3, 98, 4)
+    im_context_class->set_client_widget = ibus_im_context_set_client_widget;
+#else
     im_context_class->set_client_window = ibus_im_context_set_client_window;
+#endif
     im_context_class->set_cursor_location = ibus_im_context_set_cursor_location;
     im_context_class->set_use_preedit = ibus_im_context_set_use_preedit;
+#if GTK_CHECK_VERSION (4, 1, 2)
+    im_context_class->set_surrounding_with_selection
+            = ibus_im_context_set_surrounding_with_selection;
+#else
     im_context_class->set_surrounding = ibus_im_context_set_surrounding;
+#endif
     gobject_class->notify = ibus_im_context_notify;
     gobject_class->finalize = ibus_im_context_finalize;
 
@@ -630,9 +776,13 @@ ibus_im_context_class_init (IBusIMContextClass *class)
         g_signal_lookup ("retrieve-surrounding", G_TYPE_FROM_CLASS (class));
     g_assert (_signal_retrieve_surrounding_id != 0);
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+    _use_sync_mode = _get_boolean_env ("IBUS_ENABLE_SYNC_MODE", TRUE);
+#else
     _use_key_snooper = !_get_boolean_env ("IBUS_DISABLE_SNOOPER",
                                           !(ENABLE_SNOOPER));
     _use_sync_mode = _get_boolean_env ("IBUS_ENABLE_SYNC_MODE", FALSE);
+#endif
     _use_discard_password = _get_boolean_env ("IBUS_DISCARD_PASSWORD", FALSE);
 
 #define CHECK_APP_IN_CSV_ENV_VARIABLES(retval,                          \
@@ -656,6 +806,7 @@ ibus_im_context_class_init (IBusIMContextClass *class)
     g_strfreev (apps);                                                  \
 }
 
+#if !GTK_CHECK_VERSION (3, 98, 4)
     /* env IBUS_DISABLE_SNOOPER does not exist */
     if (_use_key_snooper) {
         /* disable snooper if app is in _no_snooper_apps */
@@ -664,6 +815,7 @@ ibus_im_context_class_init (IBusIMContextClass *class)
                                         _no_snooper_apps,
                                         FALSE);
     }
+#endif
     if (!_use_discard_password) {
         CHECK_APP_IN_CSV_ENV_VARIABLES (_use_discard_password,
                                         IBUS_DISCARD_PASSWORD_APPS,
@@ -686,6 +838,7 @@ ibus_im_context_class_init (IBusIMContextClass *class)
     }
 
 
+#if !GTK_CHECK_VERSION (3, 98, 4)
     /* always install snooper */
     if (_key_snooper_id == 0) {
 #pragma GCC diagnostic push
@@ -693,6 +846,7 @@ ibus_im_context_class_init (IBusIMContextClass *class)
         _key_snooper_id = gtk_key_snooper_install (_key_snooper_cb, NULL);
 #pragma GCC diagnostic pop
     }
+#endif
 
     _daemon_name_watch_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
                                               ibus_bus_get_service_name (_bus),
@@ -706,6 +860,7 @@ ibus_im_context_class_init (IBusIMContextClass *class)
 static void
 ibus_im_context_class_fini (IBusIMContextClass *class)
 {
+#if !GTK_CHECK_VERSION (3, 98, 4)
     if (_key_snooper_id != 0) {
         IDEBUG ("snooper is terminated.");
 #pragma GCC diagnostic push
@@ -714,6 +869,7 @@ ibus_im_context_class_fini (IBusIMContextClass *class)
 #pragma GCC diagnostic pop
         _key_snooper_id = 0;
     }
+#endif
 
     g_bus_unwatch_name (_daemon_name_watch_id);
 }
@@ -849,7 +1005,11 @@ ibus_im_context_finalize (GObject *obj)
         ibus_proxy_destroy ((IBusProxy *)ibusimcontext->ibuscontext);
     }
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+    ibus_im_context_set_client_widget ((GtkIMContext *)ibusimcontext, NULL);
+#else
     ibus_im_context_set_client_window ((GtkIMContext *)ibusimcontext, NULL);
+#endif
 
     if (ibusimcontext->slave) {
         g_object_unref (ibusimcontext->slave);
@@ -864,8 +1024,13 @@ ibus_im_context_finalize (GObject *obj)
         pango_attr_list_unref (ibusimcontext->preedit_attrs);
     }
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+    g_queue_free_full (ibusimcontext->events_queue,
+                       (GDestroyNotify)gdk_event_unref);
+#else
     g_queue_free_full (ibusimcontext->events_queue,
                        (GDestroyNotify)gdk_event_free);
+#endif
 
     G_OBJECT_CLASS(parent_class)->finalize (obj);
 }
@@ -902,7 +1067,11 @@ ibus_im_context_clear_preedit_text (IBusIMContext *ibusimcontext)
 
 static gboolean
 ibus_im_context_filter_keypress (GtkIMContext *context,
+#if GTK_CHECK_VERSION (3, 98, 4)
+                                 GdkEvent     *event)
+#else
                                  GdkEventKey  *event)
+#endif
 {
     IDEBUG ("%s", __FUNCTION__);
 
@@ -917,6 +1086,15 @@ ibus_im_context_filter_keypress (GtkIMContext *context,
     if (!ibusimcontext->has_focus)
         return gtk_im_context_filter_keypress (ibusimcontext->slave, event);
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+    {
+        GdkModifierType state = gdk_event_get_modifier_state (event);
+        if (state & IBUS_HANDLED_MASK)
+            return TRUE;
+        if (state & IBUS_IGNORED_MASK)
+            return ibus_im_context_commit_event (ibusimcontext, event);
+    }
+#else
     if (event->state & IBUS_HANDLED_MASK)
         return TRUE;
 
@@ -931,17 +1109,28 @@ ibus_im_context_filter_keypress (GtkIMContext *context,
     if (ibusimcontext->client_window == NULL && event->window != NULL)
         gtk_im_context_set_client_window ((GtkIMContext *)ibusimcontext,
                                           event->window);
+#endif
 
     _request_surrounding_text (ibusimcontext);
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+    ibusimcontext->time = gdk_event_get_time (event);
+    ibusimcontext->surface= gdk_event_get_surface (event);
+    ibusimcontext->device = gdk_event_get_device (event);
+    gdk_event_get_position (event, &ibusimcontext->x, &ibusimcontext->y);
+#else
     ibusimcontext->time = event->time;
+#endif
 
     if (ibusimcontext->ibuscontext) {
-        if (_process_key_event (ibusimcontext->ibuscontext, event))
+        if (_process_key_event (ibusimcontext->ibuscontext,
+                                event,
+                                ibusimcontext)) {
             return TRUE;
-        else
+        } else {
             return gtk_im_context_filter_keypress (ibusimcontext->slave,
                                                    event);
+        }
     }
 
     /* At this point we _should_ be waiting for the IBus context to be
@@ -952,12 +1141,21 @@ ibus_im_context_filter_keypress (GtkIMContext *context,
                           ibus_bus_is_connected (_bus) == FALSE,
                           FALSE);
     g_queue_push_tail (ibusimcontext->events_queue,
+#if GTK_CHECK_VERSION (3, 98, 4)
+                       gdk_event_ref (event));
+#else
                        gdk_event_copy ((GdkEvent *)event));
+#endif
 
     if (g_queue_get_length (ibusimcontext->events_queue) > MAX_QUEUED_EVENTS) {
         g_warning ("Events queue growing too big, will start to drop.");
+#if GTK_CHECK_VERSION (3, 98, 4)
+        gdk_event_unref ((GdkEvent *)
+                         g_queue_pop_head (ibusimcontext->events_queue));
+#else
         gdk_event_free ((GdkEvent *)
                         g_queue_pop_head (ibusimcontext->events_queue));
+#endif
     }
 
     return TRUE;
@@ -966,26 +1164,29 @@ ibus_im_context_filter_keypress (GtkIMContext *context,
 static void
 ibus_im_context_focus_in (GtkIMContext *context)
 {
-    IDEBUG ("%s", __FUNCTION__);
-
     IBusIMContext *ibusimcontext = (IBusIMContext *) context;
+    GtkWidget *widget = NULL;
+
+    IDEBUG ("%s", __FUNCTION__);
 
     if (ibusimcontext->has_focus)
         return;
 
     /* don't set focus on password entry */
+#if GTK_CHECK_VERSION (3, 98, 4)
+    widget = ibusimcontext->client_window;
+#else
     if (ibusimcontext->client_window != NULL) {
-        GtkWidget *widget;
-
         gdk_window_get_user_data (ibusimcontext->client_window,
                                   (gpointer *)&widget);
 
-        if (GTK_IS_ENTRY (widget) &&
-            !gtk_entry_get_visibility (GTK_ENTRY (widget))) {
-            return;
-        }
     }
+#endif
 
+    if (widget && GTK_IS_ENTRY (widget) &&
+        !gtk_entry_get_visibility (GTK_ENTRY (widget))) {
+        return;
+    }
     /* Do not call gtk_im_context_focus_out() here.
      * google-chrome's notification popup window (Pushbullet)
      * takes the focus and the popup window disappears.
@@ -1014,10 +1215,10 @@ ibus_im_context_focus_in (GtkIMContext *context)
 
     /* set_cursor_location_internal() will get origin from X server,
      * it blocks UI. So delay it to idle callback. */
-    gdk_threads_add_idle_full (G_PRIORITY_DEFAULT_IDLE,
-                               (GSourceFunc) _set_cursor_location_internal,
-                               g_object_ref (ibusimcontext),
-                               (GDestroyNotify) g_object_unref);
+    g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                     (GSourceFunc) _set_cursor_location_internal,
+                     g_object_ref (ibusimcontext),
+                     (GDestroyNotify) g_object_unref);
 
     /* retrieve the initial surrounding-text (regardless of whether
      * the current IBus engine needs surrounding-text) */
@@ -1076,6 +1277,8 @@ ibus_im_context_reset (GtkIMContext *context)
          * IBus uses button-press-event instead until GTK is fixed.
          * https://gitlab.gnome.org/GNOME/gtk/issues/1534
          */
+        if (_use_sync_mode)
+            ibus_im_context_clear_preedit_text (ibusimcontext);
         ibus_input_context_reset (ibusimcontext->ibuscontext);
     }
     gtk_im_context_reset (ibusimcontext->slave);
@@ -1122,7 +1325,7 @@ ibus_im_context_get_preedit_string (GtkIMContext   *context,
 }
 
 
-#if !GTK_CHECK_VERSION (3, 93, 0)
+#if !GTK_CHECK_VERSION (3, 98, 4)
 /* Use the button-press-event signal until GtkIMContext always emits the reset
  * signal.
  * https://gitlab.gnome.org/GNOME/gtk/merge_requests/460
@@ -1171,8 +1374,15 @@ _connect_button_press_event (IBusIMContext *ibusimcontext,
 }
 #endif
 
+#if GTK_CHECK_VERSION (3, 98, 4)
 static void
-ibus_im_context_set_client_window (GtkIMContext *context, GdkWindow *client)
+ibus_im_context_set_client_widget (GtkIMContext *context,
+                                   GtkWidget    *client)
+#else
+static void
+ibus_im_context_set_client_window (GtkIMContext *context,
+                                   GdkWindow    *client)
+#endif
 {
     IBusIMContext *ibusimcontext;
 
@@ -1181,8 +1391,8 @@ ibus_im_context_set_client_window (GtkIMContext *context, GdkWindow *client)
     ibusimcontext = IBUS_IM_CONTEXT (context);
 
     if (ibusimcontext->client_window) {
-#if !GTK_CHECK_VERSION (3, 93, 0)
-        if (ibusimcontext->use_button_press_event)
+#if !GTK_CHECK_VERSION (3, 98, 4)
+        if (ibusimcontext->use_button_press_event && !_use_sync_mode)
             _connect_button_press_event (ibusimcontext, FALSE);
 #endif
         g_object_unref (ibusimcontext->client_window);
@@ -1191,26 +1401,41 @@ ibus_im_context_set_client_window (GtkIMContext *context, GdkWindow *client)
 
     if (client != NULL) {
         ibusimcontext->client_window = g_object_ref (client);
-#if !GTK_CHECK_VERSION (3, 93, 0)
-        if (!ibusimcontext->use_button_press_event)
+#if !GTK_CHECK_VERSION (3, 98, 4)
+        if (!ibusimcontext->use_button_press_event && !_use_sync_mode)
             _connect_button_press_event (ibusimcontext, TRUE);
 #endif
     }
+#if GTK_CHECK_VERSION (3, 98, 4)
+    if (ibusimcontext->slave)
+        gtk_im_context_set_client_widget (ibusimcontext->slave, client);
+#else
     if (ibusimcontext->slave)
         gtk_im_context_set_client_window (ibusimcontext->slave, client);
+#endif
 }
 
 static void
 _set_rect_scale_factor_with_window (GdkRectangle *area,
+#if GTK_CHECK_VERSION (3, 98, 4)
+                                    GtkWidget    *window)
+#else
                                     GdkWindow    *window)
+#endif
 {
 #if GTK_CHECK_VERSION (3, 10, 0)
     int scale_factor;
 
     g_assert (area);
+#if GTK_CHECK_VERSION (3, 98, 4)
+    g_assert (GTK_IS_WIDGET (window));
+
+    scale_factor = gtk_widget_get_scale_factor (window);
+#else
     g_assert (GDK_IS_WINDOW (window));
 
     scale_factor = gdk_window_get_scale_factor (window);
+#endif
     area->x *= scale_factor;
     area->y *= scale_factor;
     area->width *= scale_factor;
@@ -1222,6 +1447,9 @@ static gboolean
 _set_cursor_location_internal (IBusIMContext *ibusimcontext)
 {
     GdkRectangle area;
+#if GTK_CHECK_VERSION (3, 98, 4)
+    GtkWidget *root;
+#endif
 
     if(ibusimcontext->client_window == NULL ||
        ibusimcontext->ibuscontext == NULL) {
@@ -1231,6 +1459,26 @@ _set_cursor_location_internal (IBusIMContext *ibusimcontext)
     area = ibusimcontext->cursor_area;
 
 #ifdef GDK_WINDOWING_WAYLAND
+#if GTK_CHECK_VERSION (3, 98, 4)
+    root = GTK_WIDGET (gtk_widget_get_root (ibusimcontext->client_window));
+     /* FIXME: GTK_STYLE_CLASS_TITLEBAR is available in GTK3 but not GTK4.
+      * gtk_css_boxes_get_content_rect() is available in GTK4 but it's an
+      * internal API and calculate the window edge 32 in GTK3.
+      */
+    area.y += 32;
+    area.width = 50; /* FIXME: Why 50 meets the cursor position? */
+    area.height = gtk_widget_get_height (root);
+    area.height += 32;
+    if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ())) {
+        ibus_input_context_set_cursor_location_relative (
+            ibusimcontext->ibuscontext,
+            area.x,
+            area.y,
+            area.width,
+            area.height);
+        return FALSE;
+    }
+#else
     if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ())) {
         gdouble px, py;
         GdkWindow *parent;
@@ -1254,22 +1502,53 @@ _set_cursor_location_internal (IBusIMContext *ibusimcontext)
         return FALSE;
     }
 #endif
+#endif
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+#elif GTK_CHECK_VERSION (2, 91, 0)
     if (area.x == -1 && area.y == -1 && area.width == 0 && area.height == 0) {
-#if GTK_CHECK_VERSION (2, 91, 0)
         area.x = 0;
         area.y += gdk_window_get_height (ibusimcontext->client_window);
+    }
 #else
+    if (area.x == -1 && area.y == -1 && area.width == 0 && area.height == 0) {
         gint w, h;
         gdk_drawable_get_size (ibusimcontext->client_window, &w, &h);
         area.y += h;
         area.x = 0;
-#endif
     }
+#endif
 
+#if GTK_CHECK_VERSION (3, 98, 4)
+#if defined(GDK_WINDOWING_X11)
+    GdkDisplay *display = gtk_widget_get_display (ibusimcontext->client_window);
+    if (GDK_IS_X11_DISPLAY (display)) {
+        Display *xdisplay = gdk_x11_display_get_xdisplay (display);
+        Window root_window = gdk_x11_display_get_xrootwindow (display);
+        GtkNative *native = gtk_widget_get_native (
+                ibusimcontext->client_window);
+        GdkSurface *surface = gtk_native_get_surface (native);
+        /* The window is the toplevel window but not the inner text widget.
+         * Unfortunatelly GTK4 cannot get the coordinate of the text widget.
+         */
+        Window window = gdk_x11_surface_get_xid (surface);
+        Window child;
+        int x, y;
+        XTranslateCoordinates (xdisplay, window, root_window,
+                               0, 0, &x, &y, &child);
+        XWindowAttributes xwa;
+        XGetWindowAttributes (xdisplay, window, &xwa);
+        area.x = x - xwa.x + area.x;
+        area.y = y - xwa.y + area.y;
+        area.width = 50; /* FIXME: Why 50 meets the cursor position? */
+        area.height = xwa.height;
+    }
+#endif
+#else
     gdk_window_get_root_coords (ibusimcontext->client_window,
                                 area.x, area.y,
                                 &area.x, &area.y);
+#endif
     _set_rect_scale_factor_with_window (&area, ibusimcontext->client_window);
     ibus_input_context_set_cursor_location (ibusimcontext->ibuscontext,
                                             area.x,
@@ -1286,12 +1565,17 @@ ibus_im_context_set_cursor_location (GtkIMContext *context, GdkRectangle *area)
 
     IBusIMContext *ibusimcontext = IBUS_IM_CONTEXT (context);
 
+#if !GTK_CHECK_VERSION (3, 93, 0)
+    /* The area is the relative coordinates and this has to get the absolute
+     * ones in _set_cursor_location_internal() since GTK 4.0.
+     */
     if (ibusimcontext->cursor_area.x == area->x &&
         ibusimcontext->cursor_area.y == area->y &&
         ibusimcontext->cursor_area.width == area->width &&
         ibusimcontext->cursor_area.height == area->height) {
         return;
     }
+#endif
     ibusimcontext->cursor_area = *area;
     _set_cursor_location_internal (ibusimcontext);
     gtk_im_context_set_cursor_location (ibusimcontext->slave, area);
@@ -1326,7 +1610,11 @@ get_selection_anchor_point (IBusIMContext *ibusimcontext,
     if (ibusimcontext->client_window == NULL) {
         return cursor_pos;
     }
+#if GTK_CHECK_VERSION (3, 98, 4)
+    widget = ibusimcontext->client_window;
+#else
     gdk_window_get_user_data (ibusimcontext->client_window, (gpointer *)&widget);
+#endif
 
     if (!GTK_IS_TEXT_VIEW (widget)){
         return cursor_pos;
@@ -1377,11 +1665,27 @@ get_selection_anchor_point (IBusIMContext *ibusimcontext,
     return anchor;
 }
 
+#if !GTK_CHECK_VERSION (4, 1, 2)
 static void
 ibus_im_context_set_surrounding (GtkIMContext  *context,
                                  const gchar   *text,
-                                 gint           len,
-                                 gint           cursor_index)
+                                 int            len,
+                                 int            cursor_index)
+{
+    ibus_im_context_set_surrounding_with_selection (context,
+                                                    text,
+                                                    len,
+                                                    cursor_index,
+                                                    cursor_index);
+}
+#endif
+
+static void
+ibus_im_context_set_surrounding_with_selection (GtkIMContext  *context,
+                                                const gchar   *text,
+                                                int            len,
+                                                int            cursor_index,
+                                                int            anchor_index)
 {
     g_return_if_fail (context != NULL);
     g_return_if_fail (IBUS_IS_IM_CONTEXT (context));
@@ -1403,18 +1707,26 @@ ibus_im_context_set_surrounding (GtkIMContext  *context,
         ibustext = ibus_text_new_from_string (p);
         g_free (p);
 
-        guint anchor_pos = get_selection_anchor_point (ibusimcontext,
-                                                       cursor_pos,
-                                                       utf8_len);
+        gint anchor_pos = get_selection_anchor_point (ibusimcontext,
+                                                      cursor_pos,
+                                                      utf8_len);
         ibus_input_context_set_surrounding_text (ibusimcontext->ibuscontext,
                                                  ibustext,
                                                  cursor_pos,
                                                  anchor_pos);
     }
+#if GTK_CHECK_VERSION (4, 1, 2)
+    gtk_im_context_set_surrounding_with_selection (ibusimcontext->slave,
+                                                   text,
+                                                   len,
+                                                   cursor_index,
+                                                   anchor_index);
+#else
     gtk_im_context_set_surrounding (ibusimcontext->slave,
                                     text,
                                     len,
                                     cursor_index);
+#endif
 }
 
 static void
@@ -1440,6 +1752,7 @@ _ibus_context_commit_text_cb (IBusInputContext *ibuscontext,
     _request_surrounding_text (ibusimcontext);
 }
 
+#if !GTK_CHECK_VERSION (3, 98, 4)
 static gboolean
 _key_is_modifier (guint keyval)
 {
@@ -1571,7 +1884,11 @@ _create_gdk_event (IBusIMContext *ibusimcontext,
         if (event->state & GDK_CONTROL_MASK) {
             if ((c >= '@' && c < '\177') || c == ' ') c &= 0x1F;
             else if (c == '2') {
+#if GLIB_CHECK_VERSION (2, 68, 0)
+                event->string = g_memdup2 ("\0\0", 2);
+#else
                 event->string = g_memdup ("\0\0", 2);
+#endif
                 event->length = 1;
                 buf[0] = '\0';
                 goto out;
@@ -1615,6 +1932,7 @@ _create_gdk_event (IBusIMContext *ibusimcontext,
 out:
     return event;
 }
+#endif
 
 static void
 _ibus_context_forward_key_event_cb (IBusInputContext  *ibuscontext,
@@ -1624,9 +1942,49 @@ _ibus_context_forward_key_event_cb (IBusInputContext  *ibuscontext,
                                     IBusIMContext     *ibusimcontext)
 {
     IDEBUG ("%s", __FUNCTION__);
+#if GTK_CHECK_VERSION (3, 98, 4)
+    int group = 0;
+    g_return_if_fail (GTK_IS_IM_CONTEXT (ibusimcontext));
+    if (keycode != 0) {
+        keycode += 8; // to GTK keycode
+    } else if (ibusimcontext->client_window) {
+        GdkDisplay *display =
+                gtk_widget_get_display (ibusimcontext->client_window);
+        GdkKeymapKey *keys = NULL;
+        gint n_keys = 0;
+        if (gdk_display_map_keyval (display, keyval, &keys, &n_keys)) {
+            keycode = keys->keycode;
+            group = keys->group;
+            g_free (keys);
+        } else {
+            g_warning ("Failed to parse keycode from keyval %x", keyval);
+        }
+    }
+    gtk_im_context_filter_key (
+        GTK_IM_CONTEXT (ibusimcontext),
+        (state & IBUS_RELEASE_MASK) ? FALSE : TRUE,
+        ibusimcontext->surface,
+        ibusimcontext->device,
+        ibusimcontext->time,
+        keycode,
+        (GdkModifierType)state,
+        group);
+#else
+    if (keycode == 0 && ibusimcontext->client_window) {
+        GdkDisplay *display =
+                gdk_window_get_display (ibusimcontext->client_window);
+        GdkKeymap *keymap = gdk_keymap_get_for_display (display);
+        GdkKeymapKey *keys = NULL;
+        gint n_keys = 0;
+        if (gdk_keymap_get_entries_for_keyval (keymap, keyval, &keys, &n_keys))
+            keycode = keys->keycode;
+        else
+            g_warning ("Failed to parse keycode from keyval %x", keyval);
+    }
     GdkEventKey *event = _create_gdk_event (ibusimcontext, keyval, keycode, state);
     gdk_event_put ((GdkEvent *)event);
     gdk_event_free ((GdkEvent *)event);
+#endif
 }
 
 static void
@@ -1660,9 +2018,10 @@ _ibus_context_update_preedit_text_cb (IBusInputContext  *ibuscontext,
         ibusimcontext->preedit_attrs = NULL;
     }
 
-#if !GTK_CHECK_VERSION (3, 93, 0)
+#if !GTK_CHECK_VERSION (3, 98, 4)
     if (!ibusimcontext->use_button_press_event &&
-        mode == IBUS_ENGINE_PREEDIT_COMMIT) {
+        mode == IBUS_ENGINE_PREEDIT_COMMIT &&
+        !_use_sync_mode) {
         if (ibusimcontext->client_window) {
             _connect_button_press_event (ibusimcontext, TRUE);
         }
@@ -1847,10 +2206,18 @@ _create_input_context_done (IBusBus       *bus,
         }
 
         if (!g_queue_is_empty (ibusimcontext->events_queue)) {
+#if GTK_CHECK_VERSION (3, 98, 4)
+            GdkEvent *event;
+#else
             GdkEventKey *event;
+#endif
             while ((event = g_queue_pop_head (ibusimcontext->events_queue))) {
-                _process_key_event (context, event);
+                _process_key_event (context, event, ibusimcontext);
+#if GTK_CHECK_VERSION (3, 98, 4)
+                gdk_event_unref (event);
+#else
                 gdk_event_free ((GdkEvent *)event);
+#endif
             }
         }
     }
