@@ -2,7 +2,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2013-2014 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2013-2016 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2013-2019 Takao Fujiwara <takao.fujiwara1@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,11 @@
 G_BEGIN_DECLS
 
 typedef struct _IBusComposeTable IBusComposeTable;
+typedef struct _IBusComposeTableEx IBusComposeTableEx;
+typedef struct _IBusComposeTablePrivate IBusComposeTablePrivate;
 typedef struct _IBusComposeTableCompact IBusComposeTableCompact;
+typedef struct _IBusComposeTableCompactEx IBusComposeTableCompactEx;
+typedef struct _IBusComposeTableCompactPrivate IBusComposeTableCompactPrivate;
 
 struct _IBusComposeTable
 {
@@ -39,6 +43,15 @@ struct _IBusComposeTable
     guint32 id;
 };
 
+struct _IBusComposeTableEx
+{
+    IBusComposeTablePrivate *priv;
+    guint16 *data;
+    gint max_seq_len;
+    gint n_seqs;
+    guint32 id;
+};
+
 struct _IBusComposeTableCompact
 {
     const guint16 *data;
@@ -47,15 +60,29 @@ struct _IBusComposeTableCompact
     gint n_index_stride;
 };
 
-IBusComposeTable *ibus_compose_table_new_with_file (const gchar *compose_file);
-GSList           *ibus_compose_table_list_add_array
+struct _IBusComposeTableCompactEx
+{
+    IBusComposeTableCompactPrivate *priv;
+    const guint16 *data;
+    gint max_seq_len;
+    gint n_index_size;
+    gint n_index_stride;
+};
+
+IBusComposeTableEx *
+                  ibus_compose_table_new_with_file (const gchar *compose_file);
+IBusComposeTableEx *
+                  ibus_compose_table_load_cache    (const gchar *compose_file);
+void              ibus_compose_table_save_cache    (IBusComposeTableEx
+                                                                *compose_table);
+GSList *          ibus_compose_table_list_add_array
                                                    (GSList
                                                                 *compose_tables,
                                                     const guint16
                                                                 *data,
                                                     gint         max_seq_len,
                                                     gint         n_seqs);
-GSList           *ibus_compose_table_list_add_file (GSList
+GSList *          ibus_compose_table_list_add_file (GSList
                                                                 *compose_tables,
                                                     const gchar *compose_file);
 
