@@ -1,8 +1,8 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2018 Takao Fujiwara <takao.fujiwara1@gmail.com>
- * Copyright (C) 2018 Red Hat, Inc.
+ * Copyright (C) 2018-2021 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2018-2021 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -76,11 +76,9 @@ unicode_data_new_object (UnicodeData *data)
             ibus_unicode_data_new ("code",
                                    data->code,
                                    "name",
-                                   data->name ? g_strdup (data->name)
-                                           : g_strdup (""),
+                                   data->name ? data->name : "",
                                    "alias",
-                                   data->alias ? g_strdup (data->alias)
-                                           : g_strdup (""),
+                                   data->alias ? data->alias : "",
                                    NULL);
     data->list = g_slist_append (data->list, unicode);
 }
@@ -98,8 +96,7 @@ unicode_block_new_object (UnicodeData *data)
                                     "end",
                                     data->end,
                                     "name",
-                                    data->name ? g_strdup (data->name)
-                                           : g_strdup (""),
+                                    data->name ? data->name : "",
                                    NULL);
     data->list = g_slist_append (data->list, block);
 }
@@ -285,7 +282,7 @@ ucd_parse_file (const gchar *filename,
                    filename, error ? error->message : "");
         goto failed_to_parse_ucd_names_list;
     }
-    head = end = content;
+    end = content;
     while (*end == '\n' && end - content < length) {
         end++;
         n++;
@@ -352,6 +349,7 @@ static void
 block_list_dump (IBusUnicodeBlock *block,
                  GString          *buff)
 {
+    gchar *line;
     g_return_if_fail (buff != NULL);
 
     g_string_append (buff, "    /* TRANSLATORS: You might refer the "         \
@@ -359,9 +357,10 @@ block_list_dump (IBusUnicodeBlock *block,
                            "                    the following command:\n"     \
                            "       msgmerge -C gucharmap.po ibus.po "         \
                            "ibus.pot */\n");
-    gchar *line = g_strdup_printf ("    N_(\"%s\"),\n",
-                                   ibus_unicode_block_get_name (block));
+    line = g_strdup_printf ("    N_(\"%s\"),\n",
+                            ibus_unicode_block_get_name (block));
     g_string_append (buff, line);
+    g_free (line);
 }
 
 static void
@@ -371,7 +370,7 @@ ucd_block_translatable_save (const gchar *filename,
     gchar *content = NULL;
     gsize length = 0;
     GError *error = NULL;
-    gchar *p;
+    gchar *p, *substr;
     GString *buff = NULL;
     int i;
     GSList *list = blocks_list;
@@ -392,25 +391,30 @@ ucd_block_translatable_save (const gchar *filename,
             break;
     }
     if (p != NULL) {
-        g_string_append (buff, g_strndup (content, p - content));
+        substr = g_strndup (content, p - content);
+        g_string_append (buff, substr);
+        g_free (substr);
         g_string_append_c (buff, '\n');
     }
     g_clear_pointer (&content, g_free);
 
-    g_string_append (buff, g_strdup ("\n"));
-    g_string_append (buff, g_strdup_printf ("/* This file is generated by %s. */", __FILE__));
-    g_string_append (buff, g_strdup ("\n"));
-    g_string_append (buff, g_strdup ("include <glib/gi18n.h>\n"));
-    g_string_append (buff, g_strdup ("\n"));
-    g_string_append (buff, g_strdup ("#ifndef __IBUS_UNICODE_GEN_H_\n"));
-    g_string_append (buff, g_strdup ("#define __IBUS_UNICODE_GEN_H_\n"));
-    g_string_append (buff, g_strdup ("const static char *unicode_blocks[] = {\n"));
+    g_string_append (buff, "\n");
+    substr = g_strdup_printf ("/* This file is generated by %s. */", __FILE__);
+    g_string_append (buff, substr);
+    g_free (substr);
+    g_string_append (buff, "\n");
+    g_string_append (buff, "include <glib/gi18n.h>\n");
+    g_string_append (buff, "\n");
+    g_string_append (buff, "#ifndef __IBUS_UNICODE_GEN_H_\n");
+    g_string_append (buff, "#define __IBUS_UNICODE_GEN_H_\n");
+    g_string_append (buff, "const static char *unicode_blocks[] = {\n");
     g_slist_foreach (list, (GFunc)block_list_dump, buff);
-    g_string_append (buff, g_strdup ("};\n"));
-    g_string_append (buff, g_strdup ("#endif\n"));
+    g_string_append (buff, "};\n");
+    g_string_append (buff, "#endif\n");
 
     if (!g_file_set_contents (filename, buff->str, -1, &error)) {
-        g_warning ("Failed to save emoji category file %s: %s", filename, error->message);
+        g_warning ("Failed to save emoji category file %s: %s",
+                   filename, error->message);
         g_error_free (error);
     }
 
